@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2007  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -101,9 +101,14 @@
 
 #define BIOS_VIDEO_SAVEPTR              0x4a8
 
+/* maximum of scancodes handled by keyboard bios routines */
+#define MAX_SCAN_CODE 0x58
+
 /* The Section handling Bios Disk Access */
 #define BIOS_MAX_DISK 10
 
+#define MAX_SWAPPABLE_DISKS 20
+
 struct diskGeo {
 	Bit32u ksize;  /* Size in kilobytes */
 	Bit16u secttrack; /* Sectors per track */
@@ -159,6 +164,7 @@ extern DOS_DTA *imgDTA;
 
 void swapInDisks(void);
 void swapInNextDisk(void);
+bool getSwapRequest(void);
 
 void BIOS_ZeroExtendedSize(bool in);
 void char_out(Bit8u chr,Bit32u att,Bit8u page);
@@ -169,4 +175,10 @@ void INT2F_StartUp(void);
 void INT33_StartUp(void);
 void INT13_StartUp(void);
 
+bool BIOS_AddKeyToBuffer(Bit16u code);
+
+void INT10_ReloadRomFonts();
+
+void BIOS_SetComPorts (Bit16u baseaddr[]);
+
 #endif
