@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2007  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ Bitu MEM_FreeLargest(void);			//Largest free 4 kb pages block
 Bitu MEM_TotalPages(void);			//Total amount of 4 kb pages
 Bitu MEM_AllocatedPages(MemHandle handle); // amount of allocated pages of handle
 MemHandle MEM_AllocatePages(Bitu pages,bool sequence);
+MemHandle MEM_GetNextFreePage(void);
 PhysPt MEM_AllocatePage(void);
 void MEM_ReleasePages(MemHandle handle);
 bool MEM_ReAllocatePages(MemHandle & handle,Bitu pages,bool sequence);
@@ -149,7 +150,7 @@ INLINE Bit32u phys_readd(PhysPt addr){
 
 /* These don't check for alignment, better be sure it's correct */
 
-void MEM_BlockWrite(PhysPt pt,void * data,Bitu size);
+void MEM_BlockWrite(PhysPt pt,void const * const data,Bitu size);
 void MEM_BlockRead(PhysPt pt,void * data,Bitu size);
 void MEM_BlockCopy(PhysPt dest,PhysPt src,Bitu size);
 void MEM_StrCopy(PhysPt pt,char * data,Bitu size);
