@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: dos_classes.cpp,v 1.49 2007/01/08 19:45:39 qbix79 Exp $ */
+/* $Id: dos_classes.cpp,v 1.52 2007/06/13 07:25:14 qbix79 Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -179,7 +179,7 @@ Bit16u DOS_PSP::rootpsp = 0;
 
 void DOS_PSP::MakeNew(Bit16u mem_size) {
 	/* get previous */
-	DOS_PSP prevpsp(dos.psp());
+//	DOS_PSP prevpsp(dos.psp());
 	/* Clear it first */
 	Bitu i;
 	for (i=0;i<sizeof(sPSP);i++) mem_writeb(pt+i,0);
@@ -461,9 +461,13 @@ void DOS_FCB::FileOpen(Bit8u _fhandle) {
 	sSave(sFCB,file_handle,_fhandle);
 	sSave(sFCB,cur_block,0);
 	sSave(sFCB,rec_size,128);
-	sSave(sFCB,rndm,0);
-	Bit8u temp=RealHandle(_fhandle);
-	sSave(sFCB,filesize,Files[temp]->size);
+//	sSave(sFCB,rndm,0); // breaks Jewels of darkness. 
+	Bit8u temp = RealHandle(_fhandle);
+	Bit32u size = 0;
+	Files[temp]->Seek(&size,DOS_SEEK_END);
+	sSave(sFCB,filesize,size);
+	size = 0;
+	Files[temp]->Seek(&size,DOS_SEEK_SET);
 	sSave(sFCB,time,Files[temp]->time);
 	sSave(sFCB,date,Files[temp]->date);
 }
