@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: dos_files.cpp,v 1.81 2007/01/08 21:20:23 qbix79 Exp $ */
+/* $Id: dos_files.cpp,v 1.91 2007/08/22 07:34:57 qbix79 Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -52,7 +52,7 @@ void DOS_SetDefaultDrive(Bit8u drive) {
 	if (drive<=DOS_DRIVES && ((drive<2) || Drives[drive])) dos.current_drive = drive;
 }
 
-bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
+bool DOS_MakeName(char const * const name,char * const fullname,Bit8u * drive) {
 
 	if(!name || *name == 0 || *name == ' ') {
 		/* Both \0 and space are seperators and
@@ -60,23 +60,23 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 		DOS_SetError(DOSERR_FILE_NOT_FOUND);
 		return false;
 	}
-
+	const char * name_int = name;
 	char tempdir[DOS_PATHLENGTH];
 	char upname[DOS_PATHLENGTH];
 	Bitu r,w;
 	*drive=dos.current_drive;
 	/* First get the drive */
-	if (name[1]==':') {
-		*drive=(name[0] | 0x20)-'a';
-		name+=2;
+	if (name_int[1]==':') {
+		*drive=(name_int[0] | 0x20)-'a';
+		name_int+=2;
 	}
 	if (*drive>=DOS_DRIVES || !Drives[*drive]) { 
 		DOS_SetError(DOSERR_PATH_NOT_FOUND);
 		return false; 
 	}
 	r=0;w=0;
-	while (name[r]!=0 && (r<DOS_PATHLENGTH)) {
-		Bit8u c=name[r++];
+	while (name_int[r]!=0 && (r<DOS_PATHLENGTH)) {
+		Bit8u c=name_int[r++];
 		if ((c>='a') && (c<='z')) {upname[w++]=c-32;continue;}
 		if ((c>='A') && (c<='Z')) {upname[w++]=c;continue;}
 		if ((c>='0') && (c<='9')) {upname[w++]=c;continue;}
@@ -90,10 +90,11 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 		case '!':	case '%':	case '{':	case '}':	case '`':	case '~':
 		case '_':	case '-':	case '.':	case '*':	case '?':	case '&':
 		case '\'':	case '+':	case '^':	case 246:	case 255:	case 0xa0:
+		case 0xe5:
 			upname[w++]=c;
 			break;
 		default:
-			LOG(LOG_FILES,LOG_NORMAL)("Makename encountered an illegal char %c hex:%X !",c,c);
+			LOG(LOG_FILES,LOG_NORMAL)("Makename encountered an illegal char %c hex:%X in %s!",c,c,name);
 			DOS_SetError(DOSERR_PATH_NOT_FOUND);return false;
 			break;
 		}
@@ -105,7 +106,7 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 	else fullname[0]=0;
 	Bit32u lastdir=0;Bit32u t=0;
 	while (fullname[t]!=0) {
-	if ((fullname[t]=='\\') && (fullname[t+1]!=0))lastdir=t;
+		if ((fullname[t]=='\\') && (fullname[t+1]!=0))lastdir=t;
 		t++;
 	};
 	r=0;w=0;
@@ -124,7 +125,7 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 
 			Bit32s iDown, cDots;
 			bool dots = true;
-			Bit32u templen =strlen(tempdir);
+			Bit32s templen=(Bit32s)strlen(tempdir);
 			for(iDown=0;(iDown < templen) && dots;iDown++)
 				if(tempdir[iDown] != '.')
 					dots = false;
@@ -133,7 +134,7 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 			cDots = templen - 1;
 			if(dots && (cDots > 0))
 			{
-				for(iDown=strlen(fullname)-1;iDown>=0;iDown--)
+				for(iDown=(Bit32s)strlen(fullname)-1;iDown>=0;iDown--)
 				{
 					if(fullname[iDown]=='\\' || iDown==0)
 					{
@@ -144,7 +145,7 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 					}
 				}
 				fullname[lastdir]=0;
-				Bit32u t=0;lastdir=0;
+				t=0;lastdir=0;
 				while (fullname[t]!=0) {
 					if ((fullname[t]=='\\') && (fullname[t+1]!=0))lastdir=t;
 					t++;
@@ -173,7 +174,7 @@ bool DOS_MakeName(char * name,char * fullname,Bit8u * drive) {
 	return true;	
 }
 
-bool DOS_GetCurrentDir(Bit8u drive,char * buffer) {
+bool DOS_GetCurrentDir(Bit8u drive,char * const buffer) {
 	if (drive==0) drive=DOS_GetDefaultDrive();
 	else drive--;
 	if ((drive>DOS_DRIVES) || (!Drives[drive])) {
@@ -184,7 +185,7 @@ bool DOS_GetCurrentDir(Bit8u drive,char * buffer) {
 	return true;
 }
 
-bool DOS_ChangeDir(char * dir) {
+bool DOS_ChangeDir(char const * const dir) {
 	Bit8u drive;char fulldir[DOS_PATHLENGTH];
 	if (!DOS_MakeName(dir,fulldir,&drive)) return false;
 	
@@ -197,7 +198,7 @@ bool DOS_ChangeDir(char * dir) {
 	return false;
 }
 
-bool DOS_MakeDir(char * dir) {
+bool DOS_MakeDir(char const * const dir) {
 	Bit8u drive;char fulldir[DOS_PATHLENGTH];
 	size_t len = strlen(dir);
 	if(!len || dir[len-1] == '\\') {
@@ -215,7 +216,7 @@ bool DOS_MakeDir(char * dir) {
 	return false;
 }
 
-bool DOS_RemoveDir(char * dir) {
+bool DOS_RemoveDir(char const * const dir) {
 /* We need to do the test before the removal as can not rely on
  * the host to forbid removal of the current directory.
  * We never change directory. Everything happens in the drives.
@@ -243,7 +244,7 @@ bool DOS_RemoveDir(char * dir) {
 	return false;
 }
 
-bool DOS_Rename(char * oldname,char * newname) {
+bool DOS_Rename(char const * const oldname,char const * const newname) {
 	Bit8u driveold;char fullold[DOS_PATHLENGTH];
 	Bit8u drivenew;char fullnew[DOS_PATHLENGTH];
 	if (!DOS_MakeName(oldname,fullold,&driveold)) return false;
@@ -382,13 +383,14 @@ bool DOS_CloseFile(Bit16u entry) {
 	return true;
 }
 
-static bool PathExists(char* name) {
-	char* leading = strrchr(name,'\\');
+static bool PathExists(char const * const name) {
+	const char* leading = strrchr(name,'\\');
 	if(!leading) return true;
 	char temp[CROSS_LEN];
 	strcpy(temp,name);
-	leading = strrchr(temp,'\\');
-	*leading = 0;
+	char * lead = strrchr(temp,'\\');
+	if (lead == temp) return true;
+	*lead = 0;
 	Bit8u drive;char fulldir[DOS_PATHLENGTH];
 	if (!DOS_MakeName(temp,fulldir,&drive)) return false;
 	if(!Drives[drive]->TestDir(fulldir)) return false;
@@ -396,7 +398,7 @@ static bool PathExists(char* name) {
 }
 
 
-bool DOS_CreateFile(char * name,Bit16u attributes,Bit16u * entry) {
+bool DOS_CreateFile(char const * name,Bit16u attributes,Bit16u * entry) {
 	// Creation of a device is the same as opening it
 	// Tc201 installer
 	if (DOS_FindDevice(name) != DOS_DEVICES)
@@ -437,14 +439,14 @@ bool DOS_CreateFile(char * name,Bit16u attributes,Bit16u * entry) {
 	}
 }
 
-bool DOS_OpenFile(char * name,Bit8u flags,Bit16u * entry) {
+bool DOS_OpenFile(char const * name,Bit8u flags,Bit16u * entry) {
 	/* First check for devices */
 	if (flags>2) LOG(LOG_FILES,LOG_ERROR)("Special file open command %X file %s",flags,name);
 	else LOG(LOG_FILES,LOG_NORMAL)("file open command %X file %s",flags,name);
 
 	DOS_PSP psp(dos.psp());
 	Bit16u attr = 0;
-	Bit8u devnum = DOS_FindDevice((char *)name);
+	Bit8u devnum = DOS_FindDevice(name);
 	bool device = (devnum != DOS_DEVICES);
 	if(!device && DOS_GetFileAttr(name,&attr)) {
 	//DON'T ALLOW directories to be openened.(skip test if file is device).
@@ -499,7 +501,7 @@ bool DOS_OpenFile(char * name,Bit8u flags,Bit16u * entry) {
 	}
 }
 
-bool DOS_OpenFileExtended(char *name, Bit16u flags, Bit16u createAttr, Bit16u action, Bit16u *entry, Bit16u* status)
+bool DOS_OpenFileExtended(char const * name, Bit16u flags, Bit16u createAttr, Bit16u action, Bit16u *entry, Bit16u* status)
 // FIXME: Not yet supported : Bit 13 of flags (int 0x24 on critical error
 {
 	Bit16u result = 0;
@@ -525,7 +527,7 @@ bool DOS_OpenFileExtended(char *name, Bit16u flags, Bit16u createAttr, Bit16u ac
 	return true;
 };
 
-bool DOS_UnlinkFile(char * name) {
+bool DOS_UnlinkFile(char const * const name) {
 	char fullname[DOS_PATHLENGTH];Bit8u drive;
 	if (!DOS_MakeName(name,fullname,&drive)) return false;
 	if(Drives[drive]->FileUnlink(fullname)){
@@ -536,7 +538,7 @@ bool DOS_UnlinkFile(char * name) {
 	}
 }
 
-bool DOS_GetFileAttr(char * name,Bit16u * attr) {
+bool DOS_GetFileAttr(char const * const name,Bit16u * attr) {
 	char fullname[DOS_PATHLENGTH];Bit8u drive;
 	if (!DOS_MakeName(name,fullname,&drive)) return false;
 	if (Drives[drive]->GetFileAttr(fullname,attr)) {
@@ -547,7 +549,7 @@ bool DOS_GetFileAttr(char * name,Bit16u * attr) {
 	}
 }
 
-bool DOS_SetFileAttr(char * name,Bit16u attr) 
+bool DOS_SetFileAttr(char const * const name,Bit16u attr) 
 // this function does not change the file attributs
 // it just does some tests if file is available 
 // returns false when using on cdrom (stonekeep)
@@ -562,7 +564,7 @@ bool DOS_SetFileAttr(char * name,Bit16u attr)
 	return Drives[drive]->GetFileAttr(fullname,&attrTemp);
 }
 
-bool DOS_Canonicalize(char * name,char * big) {
+bool DOS_Canonicalize(char const * const name,char * const big) {
 //TODO Add Better support for devices and shit but will it be needed i doubt it :) 
 	Bit8u drive;
 	char fullname[DOS_PATHLENGTH];
@@ -637,11 +639,11 @@ bool DOS_ForceDuplicateEntry(Bit16u entry,Bit16u newentry) {
 
 
 
-bool DOS_CreateTempFile(char * name,Bit16u * entry) {
+bool DOS_CreateTempFile(char * const name,Bit16u * entry) {
 	/* First add random crap to the end of the name and try to open */
-	/* Todo maybe check for euhm existence of the path name */
 	char * tempname;
 	tempname=name+strlen(name);
+	dos.errorcode=0;
 	do {
 		Bit32u i;
 		for (i=0;i<8;i++) {
@@ -652,7 +654,8 @@ bool DOS_CreateTempFile(char * name,Bit16u * entry) {
 			tempname[i]=(rand()%26)+'A';
 		}
 		tempname[12]=0;
-	} while (!DOS_CreateFile(name,0,entry));
+	} while ((!DOS_CreateFile(name,0,entry)) && (dos.errorcode==DOSERR_FILE_ALREADY_EXISTS));
+	if (dos.errorcode) return false;
 	return true;
 }
 
@@ -688,7 +691,7 @@ Bit8u FCB_Parsename(Bit16u seg,Bit16u offset,Bit8u parser ,char *string, Bit8u *
 			char drive[2];
 			char name[9];
 			char ext[4];
-		} part GCC_ATTRIBUTE (packed) ;
+		} GCC_ATTRIBUTE (packed) part;
 		char full[DOS_FCBNAME];
 	} fcb_name;
 #ifdef _MSC_VER
@@ -765,15 +768,15 @@ checkext:
 		index++;
 	}
 savefcb:
-	if (!hasdrive & !(parser & PARSE_DFLT_DRIVE)) fcb_name.part.drive[0]=dos.current_drive+1; 
+	if (!hasdrive & !(parser & PARSE_DFLT_DRIVE)) fcb_name.part.drive[0] = 0;
 	if (!hasname & !(parser & PARSE_BLNK_FNAME)) strcpy(fcb_name.part.name,"        ");
-	if (!hasext & !(parser & PARSE_BLNK_FEXT)) strcpy(fcb_name.part.ext,"   ");		
+	if (!hasext & !(parser & PARSE_BLNK_FEXT)) strcpy(fcb_name.part.ext,"   ");
 	fcb.SetName(fcb_name.part.drive[0],fcb_name.part.name,fcb_name.part.ext);
 	*change=(Bit8u)(string-string_begin);
 	return ret;
 }
 
-static void DTAExtendName(char * name,char * filename,char * ext) {
+static void DTAExtendName(char * const name,char * const filename,char * const ext) {
 	char * find=strchr(name,'.');
 	if (find) {
 		strcpy(ext,find+1);
@@ -878,10 +881,9 @@ Bit8u DOS_FCBRead(Bit16u seg,Bit16u offset,Bit16u recno) {
 	Bit16u toread=rec_size;
 	if (!DOS_ReadFile(fhandle,dos_copybuf,&toread)) return FCB_READ_NODATA;
 	if (toread==0) return FCB_READ_NODATA;
-	if (toread<rec_size) {
-		PhysPt fill=Real2Phys(dos.dta())+toread;
-		Bitu i=rec_size-toread;
-		for (;i>0;i--) mem_writeb(fill++,0);
+	if (toread < rec_size) { //Zero pad copybuffer to rec_size
+		Bitu i = toread;
+		while(i < rec_size) dos_copybuf[i++] = 0;
 	}
 	MEM_BlockWrite(Real2Phys(dos.dta())+recno*rec_size,dos_copybuf,rec_size);
 	if (++cur_rec>127) { cur_block++;cur_rec=0; }
@@ -927,10 +929,15 @@ Bit8u DOS_FCBRandomRead(Bit16u seg,Bit16u offset,Bit16u numRec,bool restore) {
  * random read updates old block and old record to reflect the random data
  * before the read!!!!!!!!! and the random data is not updated! (user must do this)
  * Random block read updates these fields to reflect the state after the read!
+ */
+
+/* BUG: numRec should return the amount of records read! 
+ * Not implemented yet as I'm unsure how to count on error states (partial/failed) 
  */
 
 	DOS_FCB fcb(seg,offset);
-	Bit32u random;Bit16u old_block;Bit8u old_rec;Bit8u error;
+	Bit32u random;Bit16u old_block;Bit8u old_rec;
+	Bit8u error=0;
 
 	/* Set the correct record from the random data */
 	fcb.GetRandom(random);
@@ -952,7 +959,8 @@ Bit8u DOS_FCBRandomRead(Bit16u seg,Bit16u offset,Bit16u numRec,bool restore) {
 Bit8u DOS_FCBRandomWrite(Bit16u seg,Bit16u offset,Bit16u numRec,bool restore) {
 /* see FCB_RandomRead */
 	DOS_FCB fcb(seg,offset);
-	Bit32u random;Bit16u old_block;Bit8u old_rec;Bit8u error;
+	Bit32u random;Bit16u old_block;Bit8u old_rec;
+	Bit8u error=0;
 
 	/* Set the correct record from the random data */
 	fcb.GetRandom(random);
@@ -971,13 +979,14 @@ Bit8u DOS_FCBRandomWrite(Bit16u seg,Bit16u offset,Bit16u numRec,bool restore) {
 	return error;
 }
 
-bool DOS_FCBGetFileSize(Bit16u seg,Bit16u offset,Bit16u numRec) {
+bool DOS_FCBGetFileSize(Bit16u seg,Bit16u offset) {
 	char shortname[DOS_PATHLENGTH];Bit16u entry;Bit8u handle;Bit16u rec_size;
 	DOS_FCB fcb(seg,offset);
 	fcb.GetName(shortname);
 	if (!DOS_OpenFile(shortname,0,&entry)) return false;
-	handle=RealHandle(entry);
-	Bit32u size=Files[handle]->size;
+	handle = RealHandle(entry);
+	Bit32u size = 0;
+	Files[handle]->Seek(&size,DOS_SEEK_END);
 	DOS_CloseFile(entry);fcb.GetSeqData(handle,rec_size);
 	Bit32u random=(size/rec_size);
 	if (size % rec_size) random++;
@@ -1024,7 +1033,7 @@ void DOS_FCBSetRandomRecord(Bit16u seg, Bit16u offset) {
 }
 
 
-bool DOS_FileExists(char * name) {
+bool DOS_FileExists(char const * const name) {
 	char fullname[DOS_PATHLENGTH];Bit8u drive;
 	if (!DOS_MakeName(name,fullname,&drive)) return false;
 	return Drives[drive]->FileExists(fullname);
@@ -1033,7 +1042,7 @@ bool DOS_FileExists(char * name) {
 bool DOS_GetAllocationInfo(Bit8u drive,Bit16u * _bytes_sector,Bit8u * _sectors_cluster,Bit16u * _total_clusters) {
 	if (!drive) drive=dos.current_drive;
 	else drive--;
-	if (!Drives[drive]) return false;
+	if (drive >= DOS_DRIVES || !Drives[drive]) return false;
 	Bit16u _free_clusters;
 	Drives[drive]->AllocationInfo(_bytes_sector,_sectors_cluster,_total_clusters,&_free_clusters);
 	SegSet16(ds,RealSeg(dos.tables.mediaid));
@@ -1050,8 +1059,7 @@ bool DOS_SetDrive(Bit8u drive) {
 	}
 };
 
-bool DOS_GetFileDate(Bit16u entry, Bit16u* otime, Bit16u* odate)
-{
+bool DOS_GetFileDate(Bit16u entry, Bit16u* otime, Bit16u* odate) {
 	Bit32u handle=RealHandle(entry);
 	if (handle>=DOS_FILES) {
 		DOS_SetError(DOSERR_INVALID_HANDLE);
