@@ -26,7 +26,7 @@ private:
 	bool isOpen;
 public:
 	MidiHandler_oss() : MidiHandler(),isOpen(false) {};
-	char * GetName(void) { return "oss";};
+	const char * GetName(void) { return "oss";};
 	bool Open(const char * conf) {
 		char devname[512];
 		if (conf && conf[0]) safe_strncpy(devname,conf,512);
