@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2007  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: hardware.cpp,v 1.16 2006/03/29 13:51:58 harekiet Exp $ */
+/* $Id: hardware.cpp,v 1.18 2007/01/08 19:45:40 qbix79 Exp $ */
 
 #include <dirent.h>
 #include <string.h>
@@ -37,7 +37,7 @@
 #endif
 
 static char * capturedir;
-extern char * RunningProgram;
+extern const char* RunningProgram;
 Bitu CaptureState;
 
 #define WAVE_BUF 16*1024
