@@ -31,7 +31,6 @@ void write_p3c4(Bitu port,Bitu val,Bitu iolen) {
 };
 
 void write_p3c5(Bitu port,Bitu val,Bitu iolen) {
-	if (seq(index)>0x8 && vga.s3.pll.lock!=0x6) return;
 //	LOG_MSG("SEQ WRITE reg %X val %X",seq(index),val);
 	switch(seq(index)) {
 	case 0:		/* Reset */
@@ -114,7 +113,6 @@ void write_p3c5(Bitu port,Bitu val,Bitu iolen) {
 
 Bitu read_p3c5(Bitu port,Bitu iolen) {
 //	LOG_MSG("VGA:SEQ:Read from index %2X",seq(index));
-	if (seq(index)>0x8 && vga.s3.pll.lock!=0x6) return seq(index);
 	switch(seq(index)) {
 	case 0:			/* Reset */
 		return seq(reset);
