@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: bios.cpp,v 1.66 2007/02/22 08:33:15 qbix79 Exp $ */
+/* $Id: bios.cpp,v 1.68 2007/06/12 20:22:08 c2woody Exp $ */
 
 #include "dosbox.h"
 #include "mem.h"
@@ -364,7 +364,7 @@ static Bitu INT14_Handler(void)
 {
 	if(reg_ah > 0x3 || reg_dx > 0x3) {	// 0-3 serial port functions
 										// and no more than 4 serial ports
-		LOG_MSG("BIOS INT14: Unhandled call AH=%2X DX=%4x",reg_dx);
+		LOG_MSG("BIOS INT14: Unhandled call AH=%2X DX=%4x",reg_ah,reg_dx);
 		return CBRET_NONE;
 	}
 	
@@ -981,7 +981,7 @@ void BIOS_SetComPorts(Bit16u baseaddr[]) {
 	equipmentword &= (~0x0E00);
 	equipmentword |= (portcount << 9);
 	mem_writew(BIOS_CONFIGURATION,equipmentword);
-	CMOS_SetRegister(0x14,equipmentword); //Should be updated on changes
+	CMOS_SetRegister(0x14,(Bit8u)(equipmentword&0xff)); //Should be updated on changes
 }
 
 
